import wx
from wx.lib.pubsub import pub as Publisher
import wx.lib.dialogs as dialogs

import misc
import peaksupport as ps

class FitInteractor(object):
    nb_last_sel = 0
    
    def Install(self, controller, view, modelbuttons=None):
        self.controller = controller
        self.view = view
        
        self.view.Bind(wx.EVT_TEXT, self.OnModelText, self.view.txt_model)
        #self.view.txt_model.Bind(wx.EVT_ENTER_WINDOW, self.OnModelTextFocus)
        #self.view.pan_peakbtns.Bind(wx.EVT_BUTTON, self.OnFuncButton)
        self.view.pan_peakbtns.Bind(wx.EVT_BUTTON, self.OnFuncButton)
        self.view.Bind(wx.EVT_BUTTON, self.OnClearModel, self.view.btn_modelclear)

        if modelbuttons is not None:
            for btn in modelbuttons:
                btn.Bind(wx.EVT_ENTER_WINDOW, lambda evt: self.OnFuncButtonFocus(evt, True))
                btn.Bind(wx.EVT_LEAVE_WINDOW, lambda evt: self.OnFuncButtonFocus(evt, False))

        self.view.Bind(wx.EVT_BUTTON, self.OnPickParameters, self.view.btn_pickpars)
        self.view.Bind(wx.EVT_BUTTON, self.OnLoadFromPars, self.view.btn_loadpeaks)
        self.view.Bind(wx.EVT_BUTTON, self.OnExportPars, self.view.btn_parexport)
        self.view.Bind(wx.EVT_CHOICE, self.OnExportParsChoice, self.view.ch_parexport)
        self.view.Bind(wx.EVT_BUTTON, self.OnStartFit, self.view.btn_fit)
        self.view.Bind(wx.EVT_BUTTON, self.OnStartFit, self.view.btn_fit_quick)
        self.view.Bind(wx.EVT_CHECKBOX, self.OnLimitFitRange, self.view.cb_limitfitrange)
        self.view.Bind(wx.EVT_CHECKBOX, self.OnAutostep, self.view.cb_autostep)

        Publisher.subscribe(self.OnSelectionChanged, ('selection','changed'))

        #em.eventManager.Register(self.OnGotPars, misc.EVT_GOTPARS, self.view.canvas)
        #em.eventManager.Register(self.OnGotPars, misc.EVT_GOTPARS, self.view.pargrid)
        self.view.canvas.Bind(misc.EVT_GOTPARS, self.OnGotPars)
        self.view.pargrid.Bind(misc.EVT_GOTPARS, self.OnGotPars)
        
        self.view.Bind(wx.EVT_TOGGLEBUTTON, self.OnPlaceHandles, self.view.btn_placehandles)
        self.view.Bind(wx.EVT_BUTTON, self.OnAttachWeights, self.view.btn_storeweights)
        self.view.Bind(wx.EVT_BUTTON, self.OnClearWeightsRegion, self.view.btn_clearweights)
        self.view.weightsgrid.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.OnWeightsChanged)

        Publisher.subscribe(self.OnCanvasMode, ('canvas','newmode'))
        Publisher.subscribe(self.OnPageChanged, ('notebook','pagechanged'))

    def listen_to_handles(self, listen=True):
        if listen:
            self.view.canvas.Bind(misc.EVT_HANDLES_CHANGED, self.OnHandles)
            #em.eventManager.Register(self.OnHandles, misc.EVT_HANDLES_CHANGED, self.view.canvas)
        else:
            try:
                self.view.canvas.Unbind(misc.EVT_HANDLES_CHANGED)
                #em.eventManager.DeregisterListener(self.OnHandles)
            except KeyError:
                pass

    def OnPageChanged(self, msg):
        if msg.data.GetName() == 'fitpanel':
            self.controller.page_changed(self.view.nb_fit.GetCurrentPage().GetName())
        else:
            self.controller.page_changed(msg.data.GetName())
    
    def OnCanvasMode(self, msg):
        if msg.data != 'handle':
            self.view.btn_placehandles.SetValue(False)

    def OnPlaceHandles(self, evt):
        if self.view.btn_placehandles.GetValue():
            self.controller.start_select_weights()
        else:
            self.view.canvas.SetMode(None)

    def IsSelected(self):
        """\
        Checks if this page is visible
        """
        return self.view.GetParent().GetCurrentPage() == self.view

    def OnAutostep(self, evt):
        self.view.txt_stepsize.Enable(not evt.IsChecked())
        
    def OnLimitFitRange(self, evt):
        self.controller.set_limit_fitrange(evt.IsChecked())

    def OnHandles(self, evt):
        self.controller.set_weights_regions(evt.handles)
        
    def OnAttachWeights(self, evt):
        self.controller.attach_weights()

    def OnWeightsChanged(self, evt):
        self.controller.weights_changed()

    def OnClearWeightsRegion(self, evt):
        self.controller.clear_weights()

    def OnGotPars(self, evt):
        if evt.cmd == misc.GOTPARS_DOWN:
            wx.CallAfter(self.controller.got_pars)
        if evt.cmd == misc.GOTPARS_EDIT:
            self.controller.changed_pars()
        evt.Skip()

    def OnStartFit(self, evt):
        self.controller.start_fit()

    def OnExportParsChoice(self, evt):
        self.controller.exportwhich = self.view.exportwhich
                   
    def OnExportPars(self, evt):
        self.controller.export_pars(self.view.exportwhich, self.view.exporterrors)

    def OnLoadFromPars(self, evt):
        if self.view.pan_loadpeaks.Validate():
            self.controller.load_set_from_model(self.view.loadwhich, self.view.loadrange, self.view.loadpts)

    def OnSelectionChanged(self, msg):
        self.controller.set_selection_changed(msg.data)

    def OnPickParameters(self, evt):
        self.controller.start_pick_pars()

    def OnClearModel(self, evt):
        self.view.model = ''
        self.controller.new_tokens('')
        
    def OnModelTextFocus(self, evt):
        self.controller.analyze_model()

    def OnFuncButtonFocus(self, evt, enter=True):
        """\
        displays the description and function string of the selected token button
        """
        if enter:
            name = evt.GetEventObject().GetName().split('_')[1]
            try:
                info = ps.functions[name].info
            except KeyError:
                pass
            else:
                info += '\n'+ps.functions[name].func
                self.view.peakinfo = info
                evt.Skip()
        else:
            self.view.peakinfo = ''

    def OnFuncButton(self, evt):
        """\
        adds the function token associated with the current button to the model text control.
        Equal tokens are numbered automatically.
        """
        btn = evt.GetEventObject().GetName()
        newtoken = btn.split('_')[-1]
        self.controller.add_token(newtoken)
        
    def OnModelText(self, evt):
        self.controller.new_tokens(evt.GetString())
