#!/usr/bin/env python

##     Copyright (C) 2003 Christian Kristukat (ckkart@hoc.net)     
##     This program is free software; you can redistribute it and modify
##     it under the terms of the GNU General Public License as published by
##     the Free Software Foundation; either version 2 of the License, or
##     (at your option) any later version.

##     This program is distributed in the hope that it will be useful,
##     but WITHOUT ANY WARRANTY; without even the implied warranty of
##     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##     GNU General Public License for more details.

##     You should have received a copy of the GNU General Public License
##     along with this program; if not, write to the Free Software
##     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from wx.lib.pubsub import setuparg1            # important for freezing with py2exe
from wx.lib.pubsub import pub as Publisher

import wx
from wx import xrc

import misc

class Interactor(object):
    def Install(self, controller, view):
        self.controller = controller
        self.view = view

        def menuitem(name):
           return self.view.GetMenuBar().FindItemById(xrc.XRCID(name))

        self.view.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.OnNotebookPageChanged)

        self.view.Bind(wx.EVT_MENU, self.OnNew, menuitem('men_new'))
        self.view.Bind(wx.EVT_MENU, self.OnOpen, menuitem('men_open'))
        self.view.Bind(wx.EVT_MENU, self.OnSaveAs, menuitem('men_saveas'))
        self.view.Bind(wx.EVT_MENU, self.OnSave, menuitem('men_save'))
        self.view.Bind(wx.EVT_MENU, self.OnPgSetup, menuitem('men_pgsetup'))
        self.view.Bind(wx.EVT_MENU, self.OnPrint, menuitem('men_print'))
        self.view.Bind(wx.EVT_MENU, self.OnExportBmp, menuitem('men_exportbmp'))
        self.view.Bind(wx.EVT_MENU, self.OnClose, menuitem('men_quit'))
        self.view.Bind(wx.EVT_MENU, self.OnImport, menuitem('men_import'))
        self.view.Bind(wx.EVT_MENU, self.OnExport, menuitem('men_export'))
        self.view.Bind(wx.EVT_MENU, self.OnShowDatagrid, menuitem('men_datagrid'))
        self.view.Bind(wx.EVT_MENU, self.OnShowNotes, menuitem('men_notes'))
        self.view.Bind(wx.EVT_MENU, self.OnAbout, menuitem('men_about'))

        self.view.frame_annotations.Bind(wx.EVT_CLOSE, self.OnNotesClose)
        
        self.view.Bind(wx.EVT_MENU_RANGE, self.OnFileHistory, id=wx.ID_FILE1, id2=wx.ID_FILE9)

        self.view.canvas.Bind(misc.EVT_RANGE, self.OnCanvasNewRange)

        #self.view.Bind(wx.EVT_TOGGLEBUTTON, self.OnCanvasButton)
        #self.view.Bind(wx.EVT_BUTTON, self.OnCanvasButton)
        #self.view.Bind(wx.EVT_TOGGLEBUTTON, self.OnCanvasButton)
        self.view.tb_canvas.Bind(wx.EVT_TOOL, self.OnCanvasButton)
        
        self.view.Bind(wx.EVT_TEXT, self.OnEditAnnotations, self.view.txt_annotations)
        self.view.Bind(wx.EVT_BUTTON, self.OnRunCode, self.view.btn_runcode)
        
        Publisher.subscribe(self.OnTreeSelect, ('tree','select'))
        Publisher.subscribe(self.OnTreeDelete, ('tree','delete'))
        Publisher.subscribe(self.OnTreeRename, ('tree','rename'))
        Publisher.subscribe(self.OnTreeMove, ('tree','move'))
        Publisher.subscribe(self.OnTreeHide, ('tree','hide'))
        Publisher.subscribe(self.OnTreeDuplicate, ('tree','duplicate'))
        Publisher.subscribe(self.OnTreeNewFromVisArea, ('tree','newfromvisarea'))
        Publisher.subscribe(self.OnTreeInsert, ('tree','insert'))
        Publisher.subscribe(self.OnTreeCopyToGrid, ('tree','togrid'))
        Publisher.subscribe(self.OnTreeRemFit, ('tree','remfit'))
        Publisher.subscribe(self.OnTreeRemTrafo, ('tree','remtrafo'))
        Publisher.subscribe(self.OnTreeRemWeights, ('tree','remerror'))
        Publisher.subscribe(self.OnTreeUnmask, ('tree','unmask'))
        Publisher.subscribe(self.OnAddPlot, ('tree','addplot'))
        Publisher.subscribe(self.OnTreeCopy, ('tree','copy'))
        Publisher.subscribe(self.OnTreePaste, ('tree','paste'))
        
        Publisher.subscribe(self.OnSetFromGrid, ('grid','newset'))

        Publisher.subscribe(self.OnCanvasErase, ('canvas','erase'))
        #em.eventManager.Register(self.OnGotPars, misc.EVT_GOTPARS, self.view.canvas)
        self.view.canvas.Bind(misc.EVT_GOTPARS, self.OnGotPars)
        
        Publisher.subscribe(self.OnLoadSetFromModel, ('fitctrl','loadset'))
        Publisher.subscribe(self.OnFitPars2DataGrid, ('fitctrl','parexport'))
        Publisher.subscribe(self.OnStartFit, ('fitctrl','fit'))
        Publisher.subscribe(self.OnStartPickPars, ('fitctrl','pickpars'))
        Publisher.subscribe(self.OnEditPars, ('fitctrl','editpars'))
        Publisher.subscribe(self.OnAttachWeights, ('fitctrl','attachweights'))
        Publisher.subscribe(self.OnLimitFitRange, ('fitctrl','limitfitrange'))
        Publisher.subscribe(self.OnPlot, ('fitctrl','plot'))

        Publisher.subscribe(self.OnPageChanged, ('notebook','pagechanged'))

        Publisher.subscribe(self.OnProjectModified, ('changed'))

        self.view.Bind(misc.EVT_RESULT, self.OnFitResult)

        self.view.Bind(wx.EVT_CLOSE, self.OnClose)

    def OnRunCode(self, evt):
        Publisher.sendMessage(('runcode'), self.view.txt_annotations.GetStringSelection())        

    def OnTreeCopy(self, msg):
        self.controller.set2clipboard()

    def OnTreePaste(self, msg):
        self.controller.clipboard2set()

    def OnProjectModified(self, msg):
        self.controller.project_modified = True

    def OnAbout(self, evt):
        self.view.about_dialog()

    def OnFileHistory(self, evt):
        filenum = evt.GetId() - wx.ID_FILE1
        self.controller.open_recent(filenum)

    def OnNotebookPageChanged(self, evt):
        Publisher.sendMessage(('notebook','pagechanged'), evt.GetEventObject().GetCurrentPage())

    def OnPageChanged(self, msg):
        self.controller.page_changed(msg.data.GetName())

    def OnEditAnnotations(self, evt):
        self.controller.annotations_changed(self.view.annotations)

    def OnCanvasErase(self, msg):
        self.controller.delete_points(msg.data)

    def OnPlot(self, msg):
        self.controller.update_plot()

    def OnLimitFitRange(self, msg):
        self.controller.set_limit_fitrange(msg.data)

    def OnAttachWeights(self, msg):
        self.controller.attach_weights_to_set(msg.data)

    def OnStartFit(self, msg):
        self.controller.start_fit(*msg.data)

    def OnFitResult(self, evt):
        self.controller.fit_finished(evt.result)
        
    def OnAddPlot(self, evt):
        self.controller.add_plot()
        
    def OnTreeUnmask(self, msg):
        self.controller.rem_attr('mask', only_sel=True)

    def OnTreeRemWeights(self, msg):
        self.controller.rem_attr('weights', only_sel=True)

    def OnTreeRemFit(self, msg):
        self.controller.rem_attr('mod', only_sel=True)

    def OnTreeRemTrafo(self, msg):
        self.controller.rem_attr('trafo', only_sel=True)

    def OnFitPars2DataGrid(self, msg):
        self.controller.datagrid_append(msg.data)

    def OnLoadSetFromModel(self, msg):
        model, which, xr, pts = msg.data
        self.controller.load_set_from_model(model, which, xr, pts)

    def OnGotPars(self, evt):
        mapping = {misc.GOTPARS_MOVE: 'edit',
                   misc.GOTPARS_MOVE: 'move',
                   misc.GOTPARS_DOWN: 'down',
                   misc.GOTPARS_END: 'end'}
        
        self.controller.model_updated(action = mapping[evt.cmd])
        evt.Skip()

    def OnEditPars(self, msg):
        self.controller.model_updated()
        
    def OnStartPickPars(self, msg):
        self.controller.start_pick_pars(*msg.data)

    def OnSetFromGrid(self, msg):
        self.controller.new_sets_from_grid(msg.data)

    def OnTreeCopyToGrid(self, msg):
        self.controller.selection_to_grid()
        
    def OnTreeSelect(self, msg):
        self.controller.selection = msg.data
        
    def OnTreeDelete(self, msg):
        self.controller.delete_selection(msg.data)

    def OnTreeRename(self, msg):
        plot, set, name = msg.data
        wx.CallAfter(self.controller.rename_set, name, (plot, set))

    def OnTreeMove(self, msg):
        self.controller.move_set(*msg.data)

    def OnTreeHide(self, msg):
        self.controller.hide_selection()

    def OnTreeDuplicate(self, msg):
        self.controller.duplicate_selection(msg.data)

    def OnTreeNewFromVisArea(self, msg):
        self.controller.crop_selection(msg.data)

    def OnTreeInsert(self, msg):
        self.controller.insert_plot(msg.data)

    def OnCanvasNewRange(self, evt):
        xr, yr = evt.range
        self.controller.set_plot_range(xr,yr)

    def OnCanvasButton(self, evt):
        callmap = {'btn_peaks':self.OnCanvasButtonPeaks,
                   'btn_logx':self.OnCanvasButtonLogX,
                   'btn_logy':self.OnCanvasButtonLogY,
                   'btn_style':self.OnCanvasButtonStyle,
                   'btn_zoom':self.OnCanvasButtonZoom,
                   'btn_drag':self.OnCanvasButtonDrag,
                   'btn_erase':self.OnCanvasButtonErase,
                   'btn_auto':self.OnCanvasButtonAuto,
                   'btn_autox':self.OnCanvasButtonAutoX,
                   'btn_autoy':self.OnCanvasButtonAutoY,
                   'btn_auto2fit':self.OnCanvasButtonAuto2Fit}
        callmap = [self.OnCanvasButtonLogX,
                   self.OnCanvasButtonLogY,
                   self.OnCanvasButtonStyle,
                   self.OnCanvasButtonPeaks,
                   self.OnCanvasButtonErase,
                   self.OnCanvasButtonDrag,
                   self.OnCanvasButtonZoom,
                   self.OnCanvasButtonAuto,
                   self.OnCanvasButtonAutoX,
                   self.OnCanvasButtonAutoY,
                   self.OnCanvasButtonAuto2Fit]

        tid = evt.GetEventObject().GetToolClientData(evt.GetId())
        callmap[tid](evt.GetEventObject(), evt.GetId())
        evt.Skip()

    def OnCanvasButtonAuto(self, *args):
        self.controller.autoscale()

    def OnCanvasButtonAutoX(self, *args):
        self.controller.autoscale(X=True)

    def OnCanvasButtonAutoY(self, *args):
        self.controller.autoscale(Y=True)

    def OnCanvasButtonAuto2Fit(self, *args):
        self.controller.autoscale(fit=True)
        
    def OnCanvasButtonPeaks(self, tb, id):
        state = tb.GetToolState(id)
        self.controller.app_state.show_peaks = state
        self.controller.update_plot()
        
    def OnCanvasButtonLogY(self, tb, id):
        state = tb.GetToolState(id)
        self.controller.autoscale()
        self.controller.set_logscale(None,state)
        self.controller.update_plot()
        
    def OnCanvasButtonLogX(self, tb, id):
        state = tb.GetToolState(id)
        self.controller.autoscale()
        self.controller.set_logscale(state, None)
        self.controller.update_plot()
        
    def OnCanvasButtonStyle(self, tb, id):
        state = tb.GetToolState(id)
        self.controller.app_state.line_style = state
        self.controller.update_plot()
        
    def OnCanvasButtonZoom(self, tb, id):
        state = tb.GetToolState(id)
        mode = [None,'zoom'][state]
        self.controller.set_canvas_mode(mode)
        
    def OnCanvasButtonDrag(self, tb, id):
        state = tb.GetToolState(id)
        mode = [None,'drag'][state]
        self.controller.set_canvas_mode(mode)
                
    def OnCanvasButtonErase(self, tb, id):
        state = tb.GetToolState(id)
        mode = [None,'erase'][state]
        self.controller.set_canvas_mode(mode)

    def OnImport(self, evt):
        res = self.view.import_dialog(misc.cwd())
        if res is not None:
            path,one_plot_each = res
            self.controller.import_data(path, one_plot_each)

    def OnExport(self, evt):
        self.controller.show_export_dialog()

    def OnShowDatagrid(self, evt):
        self.controller.show_datagrid(evt.IsChecked())
                
    def OnShowNotes(self, evt):
        self.controller.show_notes(evt.IsChecked())

    def OnClose(self, evt):
        self.controller.close()
        
    def OnNotesClose(self, evt):
        self.controller.notes_close()
        
    def OnNew(self, evt):
        self.controller.new_project()

    def OnOpen(self,evt):
        path = self.view.load_file_dialog(misc.cwd())
        if path is not None:
            self.controller.open_project(path)

    def OnSave(self, evt):
        if self.controller.project.path is None:
            self.OnSaveAs(None)
        else:
            self.controller.save_project()

    def OnSaveAs(self, evt):
        path = self.view.save_file_dialog(misc.cwd())
        if path is not None:
            self.controller.save_project(path)

    def OnPgSetup(self, evt):
        self.view.canvas.PageSetup()

    def OnPrint(self, evt):
        self.view.canvas.Printout()

    def OnExportBmp(self, evt):
        self.view.canvas.SaveFile()
