#!/usr/bin/python

##     Copyright (C) 2003 Christian Kristukat (ckkart@hoc.net)     
##     This program is free software; you can redistribute it and modify
##     it under the terms of the GNU General Public License as published by
##     the Free Software Foundation; either version 2 of the License, or
##     (at your option) any later version.

##     This program is distributed in the hope that it will be useful,
##     but WITHOUT ANY WARRANTY; without even the implied warranty of
##     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##     GNU General Public License for more details.

##     You should have received a copy of the GNU General Public License
##     along with this program; if not, write to the Free Software
##     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys
import os
import types
import numpy as np
from numbers import Number

from symbols import pom_globals
from appdata import configdir

def run():
    load_peaks()
    load_userfunc()

    from controller import Controller
    from interactor import Interactor
    from project import Project
    from mainframe import MainFrame

    if len(sys.argv) > 1:
        lpj_path = sys.argv[1]
    else:
        lpj_path = None


    configfolder = os.path.join(os.path.expanduser('~'),'.peak-o-mat')
    if not os.path.exists(configfolder):
        try:
            os.mkdir(configfolder)
        except (IOError, WindowsError):
            print 'unable to create config folder \'%s\''%configfolder
            
    if sys.platform == 'win32' and not hasattr(sys, 'frozen'):
        from winregistry import check, register
        if not check():
            try:
                register()
            except WindowsError:
                print 'privilege error'

    Controller(Project(),MainFrame(),Interactor(),lpj_path)

def load_peaks():
    pkg = __import__('peak_o_mat',fromlist=['peaks'])
    mod = getattr(pkg, 'peaks')
    for name in dir(mod):
        sym = getattr(mod, name)
        if type(sym) == types.FunctionType:
            pom_globals.update({name: sym})

def load_userfunc():
    sys.path.append(configdir())
    try:
        mod = __import__('userfunc',fromlist=[])
    except ImportError:
        print 'no userfunc.py'
    else:
        for name in dir(mod):
            sym = getattr(mod, name)
            if type(sym) in [types.FunctionType,np.ufunc] or isinstance(sym, Number):
                pom_globals.update({name: sym})

if __name__ == '__main__':
    run()

