﻿import sys
import traceback
import os
import re

import wx
from wx import xrc

import numpy as np
from numpy import inf, nan

import settings as config

xres_loaded = False
frozen_base = os.path.dirname(unicode(sys.executable, sys.getfilesystemencoding()))  
source_base = os.path.split(os.path.dirname(__file__))[0]

def xrc_resource():
    global xres_loaded
    if not xres_loaded:
        if hasattr(sys,"frozen") and sys.frozen == "windows_exe":
            xrcpath = os.path.join(frozen_base, 'xrc', 'peak-o-mat.xrc')
        else:
            xrcpath = os.path.join(source_base, 'peak-o-mat.xrc')
        res = xrc.XmlResource(xrcpath)
        assert res is not None
        xrc.XmlResource.Set(res)
        xres_loaded = True
    return res

class xrcctrl(object):
    def __getitem__(self, item):
        return self.FindWindowByName(item)

def get_bmp(image):
    if hasattr(sys, 'frozen'):
        imgpath = os.path.join(frozen_base, 'images', image)
    else:
        imgpath = os.path.join(source_base,'images',image)
    bmp = wx.Bitmap(imgpath)
    assert bmp is not None
    return bmp

_cwd = None
def cwd():
    global _cwd
    if _cwd is not None:
        while True:
            if os.path.exists(_cwd):
                return _cwd
            _cwd = os.path.split(_cwd)[0]
    try:
        path = os.getcwd()
    except:
        path = os.path.expanduser('~')
    return path

def set_cwd(cwd):
    global _cwd
    if cwd is not None:
        _cwd = os.path.split(os.path.abspath(cwd))[0]

_special_numbers=dict([('-1.#INF',-inf),('1.#INF',inf),
                      ('-1.#IND',nan),('-1.#IND00',nan),
                      ('1.#QNAN',nan),('1.#QNAN0',-nan)])

def _atof(x):
    if config.floating_point_is_comma:
        x = x.replace(',','.')
    if x in _special_numbers.keys():
        return _special_numbers[x]
    return np.float32(x)

def atof(x):
    try:
        tmp = np.float32(x)
    except ValueError:
        tmp = np.nan
    return tmp
        

def str2array(arg):
    for sep in ['\t',' ',None]:
        try:
            data = [[(lambda x: ['0',x][x.strip()!=''])(x) for x in line.split(sep)] for line in arg.strip().split('\n')]
        except ValueError:
            exctype, value = sys.exc_info()[:2]
            return None
        try:
            first = [float(q) for q in data [0]]
        except ValueError:
            header = data.pop(0)
        else:
            header = None
        data = [[atof(q) for q in l] for l in data]
        try:
            data = np.atleast_2d(np.array(data))
        except ValueError:
            maxlen = max([len(q) for q in data])
            for r in data:
                r.extend([0]*(maxlen-len(r)))
            data = np.atleast_2d(np.array(data))
			
        return header,data

from wx.lib import newevent

ResultEvent, EVT_RESULT = newevent.NewCommandEvent()
HandlesChangedEvent, EVT_HANDLES_CHANGED = newevent.NewCommandEvent()
ShoutEvent, EVT_SHOUT = newevent.NewCommandEvent()
ParEvent, EVT_GOTPARS = newevent.NewCommandEvent()
RangeEvent, EVT_RANGE = newevent.NewCommandEvent()

ShoutEvent.forever = False

GOTPARS_MOVE = 1
GOTPARS_DOWN = 2
GOTPARS_EDIT = 3
GOTPARS_END = 4

from threading import Thread

class WorkerThread(Thread):
    threadnum = 0
    def __init__(self, notify, fitter):
        Thread.__init__(self)
        self._notify = notify
        self._fitter = fitter
        WorkerThread.threadnum += 1
        
    def run(self):
        msg = self._fitter.run()
        event = ResultEvent(self._notify.GetId(), result=msg)
        wx.PostEvent(self._notify, event)

class PomError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)
    
